﻿using BPA.KitChen.GroupMeal.SqlSugar;
using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.AExternalPlatform.Enum;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Bom.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Dtos;
using BPA.SAAS.Manage.Application.DataBase.Dtos.Bom;
using BPA.SAAS.Manage.Application.DataBase.Interface;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using BPA.SAAS.Manage.Core.Device;
using NPOI.SS.Formula.Functions;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.AExternalPlatform.Service.Bom.Services
{
    public class BomService: IBomService, ITransient
    {  
        public BomService()
        {

        }
        /// <summary>
        /// 分页查询
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil<List<BomDto>>> GetBomPageList(BomPageInputDto inputDto)
        {
            int total = new RefAsync<int>();
            var data = SqlSugarDb.Db.Queryable<BPA_Bom>()
                .Select(a => new BomDto()
                {
                    Id = a.Id.SelectAll(),
                })
                .Mapper(x =>
                {
                    var list=SqlSugarDb.Db.Queryable<BPA_BomEntry, BPA_Batching>((a,b)=> new JoinQueryInfos(JoinType.Left, a.BatchingId == b.Id)).Select((a,b)=>new BomEntr
                    {
                        Id=a.Id,
                        BatchingId=a.BatchingId,
                        BomQty=a.BomQty,
                        BomId=a.BomId,
                        Sort=a.sort,
                        GroupId=a.GroupId,
                        BatchingName=b.Batching_Name
                    }).ToList();
                    x.BomEntr = list;
                })
                .ToPageList(inputDto.Current, inputDto.PageSize, ref total);

            return new PageUtil<List<BomDto>>()
            {
                Total = total,
                Data = data
            };
        }
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddBom(BaseRequestDto<BomInsertDto> dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var resEntity = new BPA_Bom();
                resEntity.Name = dto.DataInfo.Name;
                resEntity.IsMain = dto.DataInfo.IsMain;
                resEntity.Sort = dto.DataInfo.Sort;
                resEntity.Code = GetNumber2(8);
                var bom = await SqlSugarDb.Db.Insertable(resEntity).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                BPA_BomTypeInfo _BomTypeInfo = new();
                _BomTypeInfo.BomId = bom.Id;
                var check = SqlSugarDb.Db.Queryable<BPA_BomType>().Where(x => x.Name == dto.DataInfo.BomTypeName).First();
                if (check != null)
                {
                    _BomTypeInfo.BomTypeId = check.Id;
                }
                else
                {
                    var check1 = SqlSugarDb.Db.Queryable<BPA_BomType>().Where(x => x.Name == "默认分类").First();
                    if (check1 != null)
                    {
                        _BomTypeInfo.BomTypeId=check1.Id;
                    }
                    else
                    {
                        await SqlSugarDb.Db.Insertable(new BPA_BomType() { Name= "默认分类" }).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                    }
                }
                // resEntity.Code = GetNumber2(8);
                var res = await SqlSugarDb.Db.Insertable(_BomTypeInfo).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(ErrorCodeEnum.Code1007);
            }

        }
        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateBom(BaseRequestDto<BomUpdateDto> dto)
        {
            // 查询数据库中是否存在未删除的商品类型
            var resEntity = SqlSugarDb.Db.Queryable<BPA_Bom>().Where(it => it.IsDeleted == 0).First(it => it.Id == dto.DataInfo.Id);
            if (null == resEntity)
            {
                throw Oops.Oh(ErrorCodeEnum.Code10015);
            }
            resEntity.Name = dto.DataInfo.Name;
            resEntity.IsMain = dto.DataInfo.IsMain;
            resEntity.Sort = dto.DataInfo.Sort;

            var res = await SqlSugarDb.Db.Updateable(resEntity).ExecuteCommandAsync();
            return res > 0;
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        public async Task<bool> DeleteBom(string[] ids)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var goods = SqlSugarDb.Db.Queryable<BPA_Bom>().Where(x => ids.Contains(x.Id)).ToList();
                if (goods == null) throw Oops.Oh(ErrorCodeEnum.Code10015);
                var goodsbom = SqlSugarDb.Db.Queryable<BPA_BomEntry>().Where(x => ids.Contains(x.BomId)).ToList();
                await SqlSugarDb.Db.Deleteable(goodsbom).ExecuteCommandAsync();
                var res = await SqlSugarDb.Db.Deleteable(goods).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh("删除失败");
            }

        }
        /// <summary>
        /// 根据配方id查询配方详情
        /// </summary>
        /// <param name="bomId"></param>
        /// <returns></returns>
        public async Task<BomEntryDto> GetBomEntry(string bomId)
        {
            try
            {
                var resEntity=await SqlSugarDb.Db.Queryable<BPA_BomEntry, BPA_Bom,BPA_Batching>((a,b,c)=> new JoinQueryInfos(JoinType.Left, a.BomId == b.Id, JoinType.Left, a.BatchingId == c.Id))
                    .Select((a, b, c)=> new BomEntryDto()
                    {
                        Id=a.Id,
                        BatchingId=a.BatchingId,
                        BomQty=a.BomQty,
                        Sort=a.sort,
                        BatchingName=c.Batching_Name,
                        BomId = a.BomId,
                        BomName=b.Name
                    }).FirstAsync();
               
                return resEntity;
            }
            catch (Exception)
            {
                throw Oops.Oh(ErrorCodeEnum.Code1007);
            }
        }
        /// <summary>
        /// 添加配方详情
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddBomEntry(BomEntryInsertDto dto)
        {
            try
            {
                var resEntity = new BPA_BomEntry();
                resEntity.BatchingId = dto.BatchingId;
                resEntity.BomId = dto.BomId;
                resEntity.BomQty = dto.BomQty;
                resEntity.sort = dto.Sort;
                var res = await SqlSugarDb.Db.Insertable(resEntity).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception)
            {
                throw Oops.Oh(ErrorCodeEnum.Code1007);
            }
        }
        private string GetNumber2(int Length = 10)
        {
            byte[] buffer = Guid.NewGuid().ToByteArray();
            var ram = BitConverter.ToInt64(buffer, 0);
            var str = string.Format("{0}", ram.ToString().Substring(0, Length));
            return str;
        }
    }
}
