﻿using BPA.KitChen.GroupMeal.SqlSugar;
using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.AExternalPlatform.Enum;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Device.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Dtos;
using BPA.SAAS.Manage.Application.DataBase.Dtos.GoodsAttribute;
using BPA.SAAS.Manage.Comm.Const;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using BPA.SAAS.Manage.Core.Device;
using BPA.SAAS.Manage.Core.Product;
using Furion.DependencyInjection;
using Furion.RemoteRequest.Extensions;
using Newtonsoft.Json;
using SqlSugar;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ResponDataBase = BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto.ResponDataBase;

namespace BPA.SAAS.Manage.Application.AExternalPlatform.Service.Device.Services
{
    public class DeviceService:  IDeviceService, ITransient
    {
        private string BaseServerUrl = App.GetConfig<string>("baseurl");
        /// <summary>
        /// 分页查询
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil<List<DeviceDto>>> GetDevicePageList(DevicePageInputDto inputDto)
        {
            int total = new RefAsync<int>();
            var data =await SqlSugarDb.Db.Queryable<BPA_DeviceInfo, BPA_Product,BPA_ProductVesion>((a, b,c) =>
                new JoinQueryInfos(JoinType.Left, a.ProductId == b.Id, JoinType.Left, a.ProductVersionId == c.Id))
                .WhereIF(!string.IsNullOrEmpty(inputDto.ProductName), (a, b, c) => b.Name.Contains(inputDto.ProductName))
                  .WhereIF(!string.IsNullOrEmpty(inputDto.ProductVesion), (a, b, c) => c.Vesion.Contains(inputDto.ProductVesion))
                .Select((a, b, c) => new DeviceDto()
                {
                    Id = a.Id.SelectAll(),
                    ProductName = b.Name,
                    ProductVesion = c.Vesion

                }).ToPageListAsync(inputDto.Current, inputDto.PageSize,  total);

            return new PageUtil<List<DeviceDto>>()
            {
                Total = total,
                Data = data
            };
        }
        /// <summary>
        /// 添加
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddDevice(DeviceInsertDto dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var resEntity = new BPA_DeviceInfo();
                resEntity.DeviceName = dto.DeviceName;
                await CheckInsertParm(dto);//验证参数
                resEntity.StopId = dto.StopId;
                resEntity.ProductId = dto.ProductId;
                resEntity.ProductVersionId = dto.ProductVersionId;
                var check = SqlSugarDb.Db.Queryable<BPA_DeviceType>().Where(x => x.Name == dto.ProductCode).First();
                if (check != null)
                {
                    resEntity.DeviceTypeId = check.Id;
                }
                else
                {
                    var check1 = SqlSugarDb.Db.Queryable<BPA_DeviceType>().Where(x => x.Name == "默认标签").First();
                    if (check1 != null)
                    {
                        resEntity.DeviceTypeId = check1.Id;
                    }
                    else
                    {
                        var DeviceType = await SqlSugarDb.Db.Insertable(new BPA_DeviceType() { Name = "默认标签" }).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                        resEntity.DeviceTypeId = DeviceType.Id;
                    }
                }
                var res = await SqlSugarDb.Db.Insertable(resEntity).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(ErrorCodeEnum.Code1007);
            }

        }
        /// <summary>
        /// 更新
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool>UpdateDevice(DeviceUpateDto dto)
        {
            var resEntity = SqlSugarDb.Db.Queryable<BPA_DeviceInfo>().Where(it => it.IsDeleted == 0).First(it => it.Id == dto.Id);
            await CheckUpdateParm(dto);//验证参数
            resEntity.DeviceName = dto.DeviceName;
            resEntity.StopId = dto.StopId;
            resEntity.ProductId = dto.ProductId;
            resEntity.ProductVersionId = dto.ProductVersionId;
            if (!string.IsNullOrWhiteSpace(dto.ProductCode))
            {
                var check = SqlSugarDb.Db.Queryable<BPA_DeviceType>().Where(x => x.Name == dto.ProductCode).First();
                if (check != null)
                {
                    resEntity.DeviceTypeId = check.Id;
                }
                else
                {
                    throw Oops.Oh(ErrorCodeEnum.Code10013);
                   
                }
            }
            else
            {
                var check = SqlSugarDb.Db.Queryable<BPA_DeviceType>().Where(x => x.Name == "默认标签").First();
                if (check != null)
                {
                    resEntity.DeviceTypeId = check.Id;
                }
                else
                {
                    var DeviceType = await SqlSugarDb.Db.Insertable(new BPA_DeviceType() { Name = "默认标签" }).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                    resEntity.DeviceTypeId = DeviceType.Id;
                }
            }
            var res = await SqlSugarDb.Db.Updateable(resEntity).ExecuteCommandAsync();
            return res > 0;
        }
        /// <summary>
        /// 删除
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        public async Task<bool> DeleteDevice(string[] ids)
        {
            var model = SqlSugarDb.Db.Queryable<BPA_DeviceInfo>().Where(x => ids.Contains(x.Id)).ToList();
            if (model == null) throw Oops.Oh(ErrorCodeEnum.Code10012);
            var res = await SqlSugarDb.Db.Deleteable(model).ExecuteCommandAsync();
            return res > 0;
        }
        /// <summary>
        /// 校验参数
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        private async Task CheckInsertParm(DeviceInsertDto dto)
        {
            if (string.IsNullOrWhiteSpace(dto.StopId)) throw Oops.Oh(ErrorCodeEnum.Code1009);
            var getStopurl = BaseServerUrl + "api/store/getbyIdstorelist_alm?id=" + dto.StopId;
            var responseStop = await getStopurl.SetHeaders(new
            {
                groupId = App.User?.FindFirst(ClaimConst.GroupId)?.Value
            }).SetHttpMethod(HttpMethod.Get).GetAsStringAsync();
            var resStop = JsonConvert.DeserializeObject<ResponDataBase>(responseStop);
            if (resStop.statusCode != "200") throw Oops.Oh("获取场景数据失败");
            var dataStop = JsonConvert.DeserializeObject<List<StopDto>>(resStop.data.ToString());
            if (dataStop.Count==0) throw Oops.Oh("获取场景数据失败");
            if(!dataStop.Where(x=>x.Id == dto.StopId).Any()) throw Oops.Oh("场景不存在");

            var checkProduct = SqlSugarDb.Db.Queryable<BPA_Product>().Where(x => x.Id == dto.ProductId).Any();
            if (!checkProduct) throw Oops.Oh(ErrorCodeEnum.Code10010);
            var checkProductVesion = SqlSugarDb.Db.Queryable<BPA_ProductVesion>().Where(x => x.Id == dto.ProductVersionId).Any();
            if (!checkProductVesion) throw Oops.Oh(ErrorCodeEnum.Code10011);
        }
        /// <summary>
        /// 校验参数
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        private async Task CheckUpdateParm(DeviceUpateDto dto)
        {
            if (string.IsNullOrWhiteSpace(dto.StopId)) throw Oops.Oh(ErrorCodeEnum.Code1009);
            var getStopurl = BaseServerUrl + "api/store/getbyIdstorelist_alm?id=" + dto.StopId;
            var responseStop = await getStopurl.SetHeaders(new
            {
                groupId = App.User?.FindFirst(ClaimConst.GroupId)?.Value
            }).SetHttpMethod(HttpMethod.Get).GetAsStringAsync();
            var resStop = JsonConvert.DeserializeObject<ResponDataBase>(responseStop);
            if (resStop.statusCode != "200") throw Oops.Oh("获取场景数据失败");
            var dataStop = JsonConvert.DeserializeObject<List<StopDto>>(resStop.data.ToString());
            if (dataStop.Count == 0) throw Oops.Oh("获取场景数据失败");
            if (!dataStop.Where(x => x.Id == dto.StopId).Any()) throw Oops.Oh("场景不存在");

            var checkProduct = SqlSugarDb.Db.Queryable<BPA_Product>().Where(x => x.Id == dto.ProductId).Any();
            if (!checkProduct) throw Oops.Oh(ErrorCodeEnum.Code10010);
            var checkProductVesion = SqlSugarDb.Db.Queryable<BPA_ProductVesion>().Where(x => x.Id == dto.ProductVersionId).Any();
            if (!checkProductVesion) throw Oops.Oh(ErrorCodeEnum.Code10011);
        }
    }
}
