﻿using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Services;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Material.Services;
using BPA.SAAS.Manage.Core.Base;
using Microsoft.AspNetCore.Components.Forms;
using Org.BouncyCastle.Crypto;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods
{
    [ApiDescriptionSettings("开放平台", Tag = "商品管理"), AllowAnonymous]
    public class GoodsServices: IDynamicApiController
    {
        private readonly IGoodsService _goodsService;
        public GoodsServices(IGoodsService goodsService)
        {
            _goodsService = goodsService;
        }

        /// <summary>
        /// 分页查询商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        [HttpPost("/api/ExternalPlatform/Goods/GetGoodsPageList")]
        public async Task<PageUtil<List<GoodsDto>>> GetGoodsPageList(GoodsPageInputDto inputDto)
        {
            return await _goodsService.GetGoodsPageList(inputDto);
        }
        /// <summary>
        /// 添加商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/ExternalPlatform/Goods/AddGoods")]
        public async Task<bool> AddGoods(BaseRequestDto<GoodsInsertDto> dto)
        {
            return await _goodsService.AddGoods(dto);
        }
        /// <summary>
        /// 更新商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/ExternalPlatform/Goods/UpdateGoods")]
        public async Task<bool> UpdateGoods(BaseRequestDto<GoodsUpdateDto> dto)
        {
            return await _goodsService.UpdateGoods(dto);
        }
        /// <summary>
        /// 删除商品
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        [HttpPost("/api/ExternalPlatform/Goods/DeleteGoods")]
        public async Task<bool> DeleteGoods(string[] ids)
        {
            return await _goodsService.DeleteGoods(ids);
        }
        /// <summary>
        ///添加商品属性
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        [HttpPost("/api/ExternalPlatform/Goods/AddGoodsAttribute")]
        public async Task<bool> AddGoodsAttribute(GoodsAttributeinsertDto dto)
        {
            return await _goodsService.AddGoodsAttribute(dto);
        }
        /// <summary>
        /// 查询商品属性
        /// </summary>
        /// <returns></returns>
        [HttpGet("/api/ExternalPlatform/Goods/GetGoodsAttributeList")]
        public async Task<List<GoodsAttributeVewDto>> GetGoodsAttributeList()
        {
            return await _goodsService.GetGoodsAttributeList();
        }
        /// <summary>
        /// 查询商品工艺
        /// </summary>
        /// <param name="goodsId"></param>
        /// <returns></returns>
        [HttpGet("/api/ExternalPlatform/Goods/GetGoodsTechnologyAction")]
        public async Task<List<GoodsTechnologyActionListViewDto>> GetGoodsTechnologyAction(string goodsId)
        {
            return await _goodsService.GetGoodsTechnologyAction(goodsId);
        }
        /// <summary>
        /// 删除商品工艺
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        [HttpGet("/api/ExternalPlatform/Goods/DeleteGoodsTechnologyAction")]
        public async Task<bool> DeleteGoodsTechnologyAction(string id)
        {
            return await _goodsService.DeleteGoodsTechnologyAction(id);
        }
    }
}
