﻿using BPA.KitChen.GroupMeal.SqlSugar;
using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.AExternalPlatform.Enum;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Dtos;
using BPA.SAAS.Manage.Application.DataBase.Dtos.GoodsTechnology;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using BPA.SAAS.Manage.Core.Device;

namespace BPA.SAAS.Manage.Application.AExternalPlatform.Service.Goods.Services
{
    public class GoodsService:  IGoodsService, ITransient
    {
        public GoodsService() 
        {

        }
        /// <summary>
        /// 分页查询商品
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil<List<GoodsDto>>> GetGoodsPageList(GoodsPageInputDto inputDto)
        {
            int total = new RefAsync<int>();
            var data = SqlSugarDb.Db.Queryable<BPA_GoodsInfo, BPA_GoodsType, BPA_GoodsUint>((a, b, c) =>
                new JoinQueryInfos(JoinType.Left, a.GoodsTypeId == b.Id,
                    JoinType.Left, a.GoodsUintId == c.Id))
                .Where((a, b, c) => a.IsDeleted == 0)
                .WhereIF(!string.IsNullOrEmpty(inputDto.GoodsTypeName), (a, b, c) => b.Name.Contains(inputDto.GoodsTypeName))
                .Select((a, b, c) => new GoodsDto()
                {
                    Id =a.Id.SelectAll() ,
                    GoodsTypeName=b.Name,
                    GoodsUnitName=c.Name

                }).ToPageList(inputDto.Current, inputDto.PageSize, ref total);

            return new PageUtil<List<GoodsDto>>()
            {
                Total = total,
                Data = data
            };
        }
        /// <summary>
        /// 添加商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> AddGoods(BaseRequestDto<GoodsInsertDto>  dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var resEntity = new BPA_GoodsInfo();
                //resEntity.GoodsTypeId = dto.DataInfo.GoodsTypeName;
                //resEntity.GoodsUintId = dto.DataInfo.GoodsUintName;
                var check = SqlSugarDb.Db.Queryable<BPA_GoodsType>().Where(x => x.Name == dto.DataInfo.GoodsTypeName).First();
                if (check != null)
                {
                    resEntity.GoodsTypeId = check.Id;
                }
                else
                {
                    var check1 = SqlSugarDb.Db.Queryable<BPA_GoodsType>().Where(x => x.Name == "默认分类").First();
                    if (check1 != null)
                    {
                        resEntity.GoodsTypeId = check1.Id;
                    }
                    else
                    {
                        var GoodsType = await SqlSugarDb.Db.Insertable(new BPA_GoodsType() { Pid = "", Name = "默认分类" }).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                        resEntity.GoodsTypeId = GoodsType.Id;
                    }
                    
                }
                var checkUint = SqlSugarDb.Db.Queryable<BPA_GoodsUint>().Where(x => x.Name == dto.DataInfo.GoodsUintName).First();
                if (checkUint != null)
                {
                    resEntity.GoodsUintId = checkUint.Id;
                }
                else
                {
                    var checkUint1 = SqlSugarDb.Db.Queryable<BPA_GoodsUint>().Where(x => x.Name == dto.DataInfo.GoodsUintName).First();
                    if (checkUint1 != null)
                    {
                        resEntity.GoodsUintId = checkUint1.Id;
                    }
                    else
                    {
                        var GoodsUint = await SqlSugarDb.Db.Insertable(new BPA_GoodsUint() { Name = "默认单位" }).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                        resEntity.GoodsUintId = GoodsUint.Id;
                    }
                }
                resEntity.Name = dto.DataInfo.Name;
                resEntity.Descritption = dto.DataInfo.Descritption;
                resEntity.ImgUrl = dto.DataInfo.ImgUrl;
                resEntity.Price = dto.DataInfo.Price;
                resEntity.IsWeigh = dto.DataInfo.IsWeigh;
                
                resEntity.IsAttrubute = true;
                resEntity.Code = GetNumber2(8);
                var res = await SqlSugarDb.Db.Insertable(resEntity).CallEntityMethod(m => m.Create()).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(ErrorCodeEnum.Code1007);
            }
           
        }
        /// <summary>
        /// 更新商品
        /// </summary>
        /// <param name="dto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateGoods(BaseRequestDto<GoodsUpdateDto> dto)
        {
            // 查询数据库中是否存在未删除的商品类型
            var resEntity = SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().Where(it => it.IsDeleted == 0).First(it => it.Id == dto.DataInfo.Id);
            if (null == resEntity)
            {
                throw Oops.Oh(ErrorCodeEnum.Code1008);
            }
            resEntity.GoodsTypeId = dto.DataInfo.GoodsTypeId;
            resEntity.Name = dto.DataInfo.Name;
            resEntity.Descritption = dto.DataInfo.Descritption;
            resEntity.ImgUrl = dto.DataInfo.ImgUrl;
            resEntity.Price = dto.DataInfo.Price;
            resEntity.IsWeigh = dto.DataInfo.IsWeigh;
            resEntity.GoodsUintId = dto.DataInfo.GoodsUintId;
           
            var res = await SqlSugarDb.Db.Updateable(resEntity).ExecuteCommandAsync();
            return res > 0;
        }
        /// <summary>
        /// 删除商品
        /// </summary>
        /// <param name="ids"></param>
        /// <returns></returns>
        public async Task<bool> DeleteGoods(string[] ids)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                var goods = SqlSugarDb.Db.Queryable<BPA_GoodsInfo>().Where(x => ids.Contains(x.Id)).ToList();
                if (goods == null) throw Oops.Oh(ErrorCodeEnum.Code1008);
                var goodsbom = SqlSugarDb.Db.Queryable<BPA_GoodsBom>().Where(x => ids.Contains( x.Goods_Id)).ToList();
                var goodsTechnology = SqlSugarDb.Db.Queryable<BPA_GoodsTechnologyAction>().Where(x => ids.Contains(x.GoodsId)).ToList();
                await SqlSugarDb.Db.Deleteable(goodsbom).ExecuteCommandAsync();
                await SqlSugarDb.Db.Deleteable(goodsTechnology).ExecuteCommandAsync();
                var res = await SqlSugarDb.Db.Deleteable(goods).ExecuteCommandAsync();
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh("删除失败");
            }

        }
       /// <summary>
       ///添加商品属性
       /// </summary>
       /// <param name="dto"></param>
       /// <returns></returns>
        public async Task<bool> AddGoodsAttribute(GoodsAttributeinsertDto dto)
        {
            try
            {
                SqlSugarDb.Db.BeginTran();
                BPA_GoodsAttribute _GoodsAttribute = new();
                _GoodsAttribute.Id = Guid.NewGuid().ToString();
                _GoodsAttribute.AttributeName = dto.AttributeName;
                var GoodsType = SqlSugarDb.Db.Queryable<BPA_GoodsType>().Where(x => x.Name == dto.GoodsTypeName).First();
                if (GoodsType == null)
                {
                    var resGoodsType = await SqlSugarDb.Db.Insertable(new BPA_GoodsType() { Pid = "", Name = "默认分类" }).CallEntityMethod(m => m.Create()).ExecuteReturnEntityAsync();
                    _GoodsAttribute.GoodsTypeId = resGoodsType.Id;
                }
                else
                {
                    _GoodsAttribute.GoodsTypeId = GoodsType.Id;
                }
                var res = await SqlSugarDb.Db.Insertable(_GoodsAttribute).ExecuteCommandAsync();
                List<BPA_GoodsAttributeValue> GoodsAttributeValueList = new();
                if (dto.GoodsAttributeValue.Count > 0)
                {
                    SqlSugarDb.Db.Deleteable<BPA_GoodsAttributeValue>().Where(x => x.GoodsAttributeId == _GoodsAttribute.Id).ExecuteCommand();
                    for (int i = 0; i < dto.GoodsAttributeValue.Count; i++)
                    {
                        BPA_GoodsAttributeValue _GoodsAttributeValue = new();
                        _GoodsAttributeValue.GoodsAttributeId = _GoodsAttribute.Id;
                        _GoodsAttributeValue.AttributeValue = dto.GoodsAttributeValue[i].AttributeValue;
                        _GoodsAttributeValue.Sort = dto.GoodsAttributeValue[i].Sort;
                        GoodsAttributeValueList.Add(_GoodsAttributeValue);
                    }
                    SqlSugarDb.Db.Insertable(GoodsAttributeValueList).ExecuteCommand();
                }
                SqlSugarDb.Db.CommitTran();
                return res > 0;
            }
            catch (Exception)
            {
                SqlSugarDb.Db.RollbackTran();
                throw Oops.Oh(ErrorCodeEnum.Code1007);
                throw;
            }
           
        }
        /// <summary>
        /// 查询商品属性
        /// </summary>
        /// <returns></returns>
        public async Task<List<GoodsAttributeVewDto>> GetGoodsAttributeList()
        {
            var res = await SqlSugarDb.Db.Queryable<BPA_GoodsAttribute>()
                .OrderBy(a => a.CreateAt, OrderByType.Desc)
                .Select(a => new GoodsAttributeVewDto
                {
                    Id = a.Id,
                    AttributeName = a.AttributeName,
                    GoodsTypeId = a.GoodsTypeId,
                    Sort = a.Sort,
                })
                .Mapper(x =>
                {
                    var list = SqlSugarDb.Db.Queryable<BPA_GoodsAttributeValue>().Where(p => p.GoodsAttributeId == x.Id).Select(x=>new GoodsAttributeValue() { AttributeValue=x.AttributeValue,Id=x.Id, GoodsAttributeId =x.GoodsAttributeId}).ToList();
                    x.AttributeValueList = list;
                })
                .ToListAsync();
            return res;
        }
        /// <summary>
        /// 查询商品工艺
        /// </summary>
        /// <param name="goodsId"></param>
        /// <returns></returns>
        public async Task<List<GoodsTechnologyActionListViewDto>> GetGoodsTechnologyAction(string goodsId)
        {
            List<GoodsTechnologyActionListViewDto> goodsTechnologyActionListViews = new List<GoodsTechnologyActionListViewDto>();
            var list = SqlSugarDb.Db.Queryable<BPA_GoodsTechnologyAction>().Where(x => x.GoodsId == goodsId)
                .Select(x => new GoodsTechnologyActionViewDto()
                {
                    Id = x.Id.SelectAll(),
                }).OrderBy(x => x.Sort, OrderByType.Asc).ToList();
            var Devicelist = await SqlSugarDb.Db.Queryable<BPA_DeviceInfo>().ToListAsync();
            var GoodsAttributeValue = await SqlSugarDb.Db.Queryable<BPA_GoodsAttributeValue>().ToListAsync();
            var sf = list.GroupBy(x => x.GoodsAttributeId).ToList();
            var sf1 = list.GroupBy(x => x.WarehousrTemplateId).ToList();
            for (int i = 0; i < sf.Count; i++)
            {
                var sd = GoodsAttributeValue?.Where(x => sf[i].Key.Contains(x.Id)).Select(x => x.AttributeValue).ToArray();
                var sdw = GoodsAttributeValue?.Where(x => sf[i].Key.Contains(x.Id)).Select(x => x.Id).ToArray();
                var nane = string.Join('-', sd);
                var s = sf[i].AsQueryable().ToList().GroupBy(x => x.DeviceId).ToList();
                for (int t = 0; t < s.Count; t++)
                {
                    GoodsTechnologyActionListViewDto item = new()
                    {
                        DeviceId = s[t].Key,
                        WarehousrTemplateId = sf1[t].Key,
                        GoodsAttributeId = string.Join(',', sdw),
                        DeviceName = Devicelist?.FirstOrDefault(x => x.Id == s[t].Key).DeviceName + "【" + nane + "】",
                        Data = s[t].AsQueryable().ToList(),
                    };
                    goodsTechnologyActionListViews.Add(item);
                }
            }

            return goodsTechnologyActionListViews;
        }
        /// <summary>
        /// 删除商品工艺
        /// </summary>
        /// <param name="id"></param>
        /// <returns></returns>
        public async Task<bool> DeleteGoodsTechnologyAction(string id)
        {
            var item = await SqlSugarDb.Db.Queryable<BPA_GoodsTechnologyAction>().Where(x => x.Id == id).FirstAsync();
            if (item != null)
            {
                var res = await SqlSugarDb.Db.Deleteable(item).ExecuteCommandAsync() > 0;
                var list = SqlSugarDb.Db.Queryable<BPA_GoodsTechnologyAction>().Where(x => x.GoodsId == item.GoodsId && x.DeviceId == item.DeviceId).OrderBy(x => x.Sort, OrderByType.Asc).ToList();
                var uplist = new List<BPA_GoodsTechnologyAction>();
                for (int i = 0; i < list.Count; i++)
                {
                    list[i].Sort = i + 1;
                    uplist.Add(list[i]);
                }
                if (uplist.Count > 0)
                {
                    await SqlSugarDb.Db.Updateable(uplist).ExecuteCommandAsync();
                }
                return true;
            }
            else
            {
                throw Oops.Oh("找不到相关数据，删除失败");
            }
        }
        private string GetNumber2(int Length = 10)
        {
            byte[] buffer = Guid.NewGuid().ToByteArray();
            var ram = BitConverter.ToInt64(buffer, 0);
            var str = string.Format("{0}", ram.ToString().Substring(0, Length));
            return str;
        }
    }
}
