﻿using BPA.KitChen.GroupMeal.SqlSugar;
using BPA.SAAS.KitChenManage.Core;
using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.AExternalPlatform.Enum;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.CheckService.Services;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Material.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.ThirdpartyPush;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.ThirdpartyPush.Dtos;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.ThirdpartyPush.Services;
using BPA.SAAS.Manage.Comm.Enum;
using BPA.SAAS.Manage.Core;
using BPA.SAAS.Manage.Core.Base;
using BPA.SAAS.Manage.Core.DataBase;
using BPA.SAAS.Manage.Core.Device;
using BPA.SAAS.Manage.Core.Product;
using Microsoft.AspNetCore.Components.Forms;
using Newtonsoft.Json;

namespace BPA.SAAS.Manage.Application.AExternalPlatform.Service.Material.Services
{
    public class MaterialService : IMaterialServices, ITransient
    {
        private readonly ICheckServices _checkServices;
        private readonly IThirdpartyPushService _thirdpartyPushService;
        public MaterialService(ICheckServices checkServices, IThirdpartyPushService thirdpartyPushService)
        {
            _checkServices = checkServices;
            _thirdpartyPushService = thirdpartyPushService;
        }

        /// <summary>
        /// 分页查询物料
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<PageUtil<List<MaterialDto>>> GetMaterialPageList(MaterialPageInputDto inputDto)
        {


            int total = new RefAsync<int>();
            var data = SqlSugarDb.Db.Queryable<BPA_Batching, BPA_BatchingType, BPA_BatchingUint>((a, b, c) =>
                new JoinQueryInfos(JoinType.Left, a.Batching_Type == b.Id,
                    JoinType.Left, a.StockUint == c.Id))
                .Where((a, b, c) => a.IsDeleted == 0)
                .WhereIF(!string.IsNullOrEmpty(inputDto.Name), (a, b, c) => a.Batching_Name.Contains(inputDto.Name))
                .Select((a, b, c) => new MaterialDto()
                {
                    Id = a.Id,
                    Code = a.Code,
                    Name = a.Batching_Name,
                    //TypeId = b.Id,
                    TypeName = b.Name,
                    // UintId = c.Id,
                    UintName = c.Name,

                }).ToPageList(inputDto.Current, inputDto.PageSize, ref total);

            return new PageUtil<List<MaterialDto>>()
            {
                Total = total,
                Data = data
            };
        }


        /// <summary>
        /// 添加物料
        /// </summary>
        /// <param name="InputDto"></param>
        /// <returns></returns>
        public async Task<bool> AddMaterial(BaseRequestDto<MaterialCreateDto> inputDto)
        {
            try
            {

                SqlSugarDb.Db.Ado.BeginTran();
                //1.物料单位查询
                var typeData = await SqlSugarDb.Db.Queryable<BPA_BatchingType>().FirstAsync(x => x.Name == inputDto.DataInfo.TypeName);
                if (typeData == null)
                {

                    typeData = new BPA_BatchingType()
                    {
                        GroupId = CurrentUser.GroupId,
                        Id = Guid.NewGuid().ToString(),
                        Name = string.IsNullOrEmpty(inputDto.DataInfo.TypeName) ? "默认分类" : inputDto.DataInfo.TypeName,
                    };
                    SqlSugarDb.Db.Insertable(typeData).ExecuteCommand();
                }

                //2.单位查询
                var uintData = await SqlSugarDb.Db.Queryable<BPA_BatchingUint>()
                    .FirstAsync(x => x.Name == inputDto.DataInfo.UintName);
                if (uintData == null)
                {
                    uintData = new BPA_BatchingUint()
                    {
                        GroupId = CurrentUser.GroupId,
                        Id = Guid.NewGuid().ToString(),
                        Name = string.IsNullOrEmpty(inputDto.DataInfo.UintName) ? "默认分类" : inputDto.DataInfo.UintName,
                    };
                    SqlSugarDb.Db.Insertable(uintData).ExecuteCommand();
                }
                //3.物料查询
                var materialData = await SqlSugarDb.Db.Queryable<BPA_Batching>().FirstAsync(x => x.Batching_Name == inputDto.DataInfo.Name);
                if (materialData == null)
                {
                    materialData = new BPA_Batching()
                    {
                        Id = Guid.NewGuid().ToString(),
                        GroupId = CurrentUser.GroupId,
                        Aittribute = 0,
                        Batching_Name = inputDto.DataInfo.Name,
                        Batching_Type = typeData.Id,
                        Code = inputDto.DataInfo.Code,
                        Price = 0,
                        Specs = "",
                        outstockUint = uintData.Id,
                        Status = CommonStatus.ENABLE,
                        StockUint = uintData.Id,
                        TypeID = typeData.Id,
                        IsDeleted = 0,
                    };

                    SqlSugarDb.Db.Insertable(materialData).ExecuteCommand();
                }
                else
                {
                    throw Oops.Oh(ErrorCodeEnum.Code1002);
                }

                var input = new List<PushDeviceDto>();


                if (inputDto.StoreIdList != null && inputDto.StoreIdList.Count > 0)
                {
                    input = await SqlSugarDb.Db.Queryable<BPA_DeviceInfo, BPA_Product,BPA_ProductVesion>((a, b,c) => new JoinQueryInfos(
                            JoinType.Left, a.ProductId == b.Id,
                            JoinType.Left,a.ProductVersionId==c.Id
                            ))
                        .Where((a, b, c) => inputDto.StoreIdList.Contains(a.StopId))
                        .Select((a, b,c) => new PushDeviceDto
                        {
                            deviceAutoKey = a.AutoKey,
                            deviceId = a.Id,
                            deviceName = a.DeviceName,
                            messageId = CurrentUser.MessageId,
                            productKey = b.Key,
                            productName = b.Name,
                            productVersion = c.Vesion,
                            type = 2

                        })
                        .ToListAsync();

                }

                if (inputDto.DeviceIdList != null && inputDto.DeviceIdList.Count > 0)
                {
                    input = await SqlSugarDb.Db.Queryable<BPA_DeviceInfo, BPA_Product, BPA_ProductVesion>((a, b,c) => new JoinQueryInfos(
                            JoinType.Left, a.ProductId == b.Id,
                            JoinType.Left,a.ProductVersionId==c.Id))
                        .Where((a, b,c) => inputDto.DeviceIdList.Contains(a.Id))
                        .Select((a, b,c) => new PushDeviceDto
                        {
                            deviceAutoKey = a.AutoKey,
                            deviceId = a.Id,
                            deviceName = a.DeviceName,
                            messageId = CurrentUser.MessageId,
                            productKey = b.Key,
                            productName = b.Name,
                            productVersion =c.Vesion,
                            type = 2,
                        })
                        .ToListAsync();
                }

                //4.添加记录
                foreach (var item in input)
                {
                    await _thirdpartyPushService.AddThirdpartyPushRecord(new Core.System.BPA_ThirdpartyPushRecord()
                    {
                        CallbackUrl = inputDto.CallbackUrl,
                        GroupId = CurrentUser.GroupId,
                        Id = Guid.NewGuid().ToString(),
                        IsPush = false,
                        MessageId = CurrentUser.MessageId,
                        PushDevice =item.deviceId,
                        PushData = materialData.Id,
                        Status=1,
                        Type = 2,
                    });
                    item.data = JsonConvert.SerializeObject(new List<PushDataBatchingDto>()
                    {
                        new()
                        {
                            Aittribute= materialData.Aittribute,
                            AutoKey= materialData.AutoKey,
                            Code=materialData.Code,
                            ForeignKeyRe= materialData.ForeignKeyRe,
                            Id=materialData.Id,
                            Name= materialData.Batching_Name,
                            netrecovery= materialData.netrecovery,
                            outstockUint= materialData.outstockUint,
                            Price= materialData.Price,
                            proportion= materialData.proportion,
                            Specs= materialData.Specs,
                            StockUint= materialData.StockUint,
                            StockUintName= materialData.StockUint,
                            TypeID= materialData.TypeID,

                        }
                    });
                }
                SqlSugarDb.Db.Ado.CommitTran();
                //5.发送mq
                await _thirdpartyPushService.PushDevice(input);
                return true;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }

        /// <summary>
        /// 删除物料
        /// </summary>
        /// <param name="MaterialId"></param>
        /// <returns></returns>
        public async Task<bool> DelMaterial(BaseRequestDto<DelMaterialDto> inputDto)
        {
            try
            {
                var materialData = await SqlSugarDb.Db.Queryable<BPA_Batching>().FirstAsync(x => x.Id == inputDto.DataInfo.MaterialId);
                if (materialData == null)
                {
                    throw Oops.Oh(ErrorCodeEnum.Code1003);
                }
                var res = await SqlSugarDb.Db.Deleteable(materialData).ExecuteCommandAsync();
                return res > 0;
            }
            catch (Exception e)
            {
                throw Oops.Oh(e.Message);
            }


        }

        /// <summary>
        /// 修改物料
        /// </summary>
        /// <param name="InputDto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateMateria(BaseRequestDto<MaterialUpdateDto> inputDto)
        {
            try
            {


                SqlSugarDb.Db.Ado.BeginTran();
                //1.物料单位查询
                var typeData = await SqlSugarDb.Db.Queryable<BPA_BatchingType>().FirstAsync(x => x.Name == inputDto.DataInfo.TypeName);
                if (typeData == null)
                {

                    typeData = new BPA_BatchingType()
                    {
                        GroupId = CurrentUser.GroupId,
                        Id = Guid.NewGuid().ToString(),
                        Name = string.IsNullOrEmpty(inputDto.DataInfo.TypeName) ? "默认分类" : inputDto.DataInfo.TypeName,
                    };
                    SqlSugarDb.Db.Insertable(typeData).ExecuteCommand();
                }

                //2.单位查询
                var uintData = await SqlSugarDb.Db.Queryable<BPA_BatchingUint>()
                    .FirstAsync(x => x.Name == inputDto.DataInfo.UintName);
                if (uintData == null)
                {
                    uintData = new BPA_BatchingUint()
                    {
                        GroupId = CurrentUser.GroupId,
                        Id = Guid.NewGuid().ToString(),
                        Name = string.IsNullOrEmpty(inputDto.DataInfo.UintName) ? "默认分类" : inputDto.DataInfo.UintName,
                    };
                    SqlSugarDb.Db.Insertable(uintData).ExecuteCommand();
                }
                //3.物料查询
                var materialData = await SqlSugarDb.Db.Queryable<BPA_Batching>().FirstAsync(x => x.Code == inputDto.DataInfo.Code);
                if (materialData == null)
                {
                    throw Oops.Oh(ErrorCodeEnum.Code1003);
                }
                else
                {
                    materialData = new BPA_Batching()
                    {
                        Id = Guid.NewGuid().ToString(),
                        GroupId = CurrentUser.GroupId,
                        Aittribute = 0,
                        Batching_Name = inputDto.DataInfo.Name,
                        Batching_Type = typeData.Id,
                        Code = inputDto.DataInfo.Code,
                        Price = 0,
                        Specs = "",
                        outstockUint = uintData.Id,
                        Status = CommonStatus.ENABLE,
                        StockUint = uintData.Id,
                        TypeID = typeData.Id,
                        IsDeleted = 0,
                    };
                    SqlSugarDb.Db.Updateable(materialData).ExecuteCommand();
                }


                SqlSugarDb.Db.Ado.CommitTran();
                return true;
            }
            catch (Exception e)
            {
                SqlSugarDb.Db.Ado.RollbackTran();
                throw Oops.Oh(e.Message);
            }
        }
    }
}
