﻿using BPA.KitChen.GroupMeal.SqlSugar;
using BPA.SAAS.KitChenManage.Core;
using BPA.SAAS.Manage.Application.AExternalPlatform.BaseDto;
using BPA.SAAS.Manage.Application.AExternalPlatform.Enum;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.Material.Services;
using BPA.SAAS.Manage.Application.AExternalPlatform.Service.ThirdpartyPush.Dtos;
using BPA.SAAS.Manage.Comm.Util;
using BPA.SAAS.Manage.Core.Device;
using BPA.SAAS.Manage.Core.System;
using Microsoft.AspNetCore.Mvc.ViewFeatures;
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace BPA.SAAS.Manage.Application.AExternalPlatform.Service.ThirdpartyPush.Services
{
    public class ThirdpartyPushService : IThirdpartyPushService, ITransient
    {
        #region 设备 回调级下发数据和记录

        /// <summary>
        /// 添加 三方 回调设备记录
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> AddThirdpartyPushRecord(BPA_ThirdpartyPushRecord inputDto)
        {
            var res = await SqlSugarDb.Db.Insertable(inputDto).ExecuteCommandAsync();

            return res > 0;

        }


        /// <summary>
        /// 下发到设备
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> PushDevice(List<PushDeviceDto> inputDto)
        {

            Dictionary<string, string> dic = new Dictionary<string, string>();
            dic.Add("groupId", CurrentUser.GroupId);
            var url = App.Configuration["baseurl"] + "api/devicepushrecode/add_alm";
            var inputData = JsonConvert.SerializeObject(inputDto);
            var jsonData = HttpHelper.PostData(url, inputData, Encoding.UTF8, "application/json", dic);
            var data = JsonConvert.DeserializeObject<ResultSAASManageDto<bool>>(jsonData);
            if (data.statusCode != 200) throw Oops.Oh(ErrorCodeEnum.Code10016);
            if (!data.data) throw Oops.Oh(ErrorCodeEnum.Code10016);
            return true;
        }


        /// <summary>
        /// 修改 第三方订单的回调
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<bool> UpdateThirdpartyPushRecord(ThirdpartyPushRecordDto inputDto)
        {
            var data = await SqlSugarDb.Db.Queryable<BPA_ThirdpartyPushRecord>()
                .ClearFilter()
                .FirstAsync(x => x.MessageId == inputDto.MessageId&&x.PushDevice==inputDto.DeviceId);
            if (data == null)
            {
                throw Oops.Oh("没有查询到回调信息");
            }
            data.IsPush = inputDto.IsPush;
            data.CreateAt = DateTime.Now;
            data.Status = 0;
            var res = await SqlSugarDb.Db.Updateable(data).ExecuteCommandAsync();

            //回调

            var urlstr = "";

            if (data.Type==2)
            {
                urlstr = $@"?materialId={data.PushData}&Device={inputDto.DeviceId}";
                Dictionary<string, string> dic = new Dictionary<string, string>();
                var inputData ="";
                 HttpHelper.PostData(urlstr, inputData, Encoding.UTF8, "application/json", dic);
            }


            return res > 0;

        }

        /// <summary>
        /// 更具店铺获取设备
        /// </summary>
        /// <param name="inputDto"></param>
        /// <returns></returns>
        public async Task<List<string>> GetDeviceByStoreId(List<string> inputDto)
        {
            var data = await SqlSugarDb.Db.Queryable<BPA_DeviceInfo>()
                .Where(x => inputDto.Contains(x.StopId))
                .Select(x => x.Id)
                .ToListAsync();
            return data;
        }



        #endregion
    }
}
